/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * This file is part of SableCC.                             *
 * See the file "LICENSE" for copyright information and the  *
 * terms and conditions for copying, distribution and        *
 * modification of SableCC.                                  *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

package org.sablecc.sablecc;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;

import org.sablecc.sablecc.analysis.DepthFirstAdapter;
import org.sablecc.sablecc.node.AAstProd;
import org.sablecc.sablecc.node.Start;

public class GenUtils extends DepthFirstAdapter
{
  private MacroExpander macros;
  private ResolveAstIds ast_ids;
  private File pkgDir;
  private String mainProduction;

  public GenUtils(ResolveAstIds ast_ids)
  {
    this.ast_ids = ast_ids;

    try
    {
      macros = new MacroExpander(
                 new InputStreamReader(
                   getClass().getResourceAsStream("utils.txt")));
    }
    catch(IOException e)
    {
      throw new RuntimeException("unable to open utils.txt.", e);
    }

    pkgDir = new File(ast_ids.astIds.pkgDir, "node");

    if(!pkgDir.exists())
    {
      if(!pkgDir.mkdir())
      {
        throw new RuntimeException("Unable to create " + pkgDir.getAbsolutePath());
      }
    }
  }

  /*
  public void caseAGrammar(AGrammar node)
  {
  AAstProd []temp = (AAstProd[])node.getAst().toArray(new AAstProd[0]);

  for(int i=0; i<temp.length; i++)
  {
    temp[i].apply(this);
  }
  }
  */

  @Override
  public void caseAAstProd(AAstProd node)
  {
    if(mainProduction == null)
    {
      mainProduction = ast_ids.ast_names.get(node);
    }
  }

  @Override
  public void outStart(Start node)
  {
    if(mainProduction != null)
    {
      createStart();
    }

    createEOF();
    createNode();
    createToken();
    create("Switch");
    create("Switchable");
  }

  public void createStart()
  {
    try(BufferedWriter file = new BufferedWriter(new FileWriter(new File(pkgDir, "Start.java"))))
    {
      macros.apply(file, "Start", new String[] {ast_ids.astIds.pkgNameDot, mainProduction, GenAlts.nodeName(mainProduction)});
    }
    catch(IOException e)
    {
      throw new RuntimeException("An error occured while writing to " +
                                 new File(pkgDir, "Start.java").getAbsolutePath(), e);
    }
  }

  public void createEOF()
  {
    try(BufferedWriter file = new BufferedWriter(new FileWriter(new File(pkgDir, "EOF.java"))))
    {
      macros.apply(file, "EOF", new String[] {ast_ids.astIds.pkgNameDot});
    }
    catch(IOException e)
    {
      throw new RuntimeException("An error occured while writing to " +
                                 new File(pkgDir, "EOF.java").getAbsolutePath(), e);
    }
  }

  public void createNode()
  {
    try(BufferedWriter file = new BufferedWriter(new FileWriter(new File(pkgDir, "Node.java"))))
    {
      macros.apply(file, "Node", new String[] {ast_ids.astIds.pkgNameDot});
    }
    catch(IOException e)
    {
      throw new RuntimeException("An error occured while writing to " +
                                 new File(pkgDir, "Node.java").getAbsolutePath(), e);
    }
  }

  public void createToken()
  {
    try(BufferedWriter file = new BufferedWriter(new FileWriter(new File(pkgDir, "Token.java"))))
    {
      macros.apply(file, "Token", new String[] {ast_ids.astIds.pkgNameDot});
    }
    catch(IOException e)
    {
      throw new RuntimeException("An error occured while writing to " +
                                 new File(pkgDir, "Token.java").getAbsolutePath(), e);
    }
  }

  public void create(String cls)
  {
    try(BufferedWriter file = new BufferedWriter(new FileWriter(new File(pkgDir, cls + ".java"))))
    {
      macros.apply(file, cls, new String[] {ast_ids.astIds.pkgNameDot});
    }
    catch(IOException e)
    {
      throw new RuntimeException("An error occured while writing to " +
                                 new File(pkgDir, cls + ".java").getAbsolutePath(), e);
    }
  }
}
